"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBulkOperationStatus = exports.getBulkQueryStatus = exports.getBulkUpdateStatus = exports.generateJSONLUploadOptions = exports.bulkUpdate = exports.bulkQuery = void 0;
const tslib_1 = require("tslib");
const api_1 = require("./api");
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const concat_stream_1 = tslib_1.__importDefault(require("concat-stream"));
const xml2js_1 = require("xml2js");
const promises_1 = require("node:stream/promises");
const node_util_1 = require("node:util");
async function bulkQuery(query) {
    const mutation = (0, api_1.gql) `
    mutation bulkOperationRunQuery($query: String!) {
      bulkOperationRunQuery(query: $query) {
        bulkOperation {
          id
          url
          status
        }
        userErrors {
          field
          message
        }
      }
    }
  `;
    return api_1.graphqlClient.request(mutation, { query });
}
exports.bulkQuery = bulkQuery;
async function bulkUpdate({ JSONLReadStream, mutation, }) {
    try {
        const JSONLUploadOptions = await generateJSONLUploadOptions();
        const form = new form_data_1.default();
        JSONLUploadOptions.parameters.forEach(({ name, value }) => {
            form.append(name, value);
        });
        form.append("file", JSONLReadStream);
        const submitForm = (0, node_util_1.promisify)(form.submit.bind(form));
        const resStream = await submitForm(JSONLUploadOptions.url);
        let uploadResMessage;
        await (0, promises_1.pipeline)(resStream, (0, concat_stream_1.default)((data) => {
            (0, xml2js_1.parseString)(data, function (err, result) {
                if (err)
                    throw err;
                uploadResMessage = result;
            });
        }));
        if (uploadResMessage?.PostResponse) {
            console.log(uploadResMessage);
            // make bulk request
            const bulkRequestResponse = await api_1.graphqlClient.request(`
             mutation {
               bulkOperationRunMutation(
                 mutation: "${mutation}",
                 stagedUploadPath: "${uploadResMessage.PostResponse.Key[0]}"
               ) {
                 bulkOperation {
                   id
                   url
                   status
                 }
                 userErrors {
                   message
                   field
                 }
               }
             }
           `);
            // console.log(bulkRequestResponse);
            return bulkRequestResponse;
        }
        else {
            throw new Error("JSONL Upload Failed");
        }
    }
    catch (e) {
        throw e;
    }
}
exports.bulkUpdate = bulkUpdate;
async function generateJSONLUploadOptions() {
    try {
        const query = (0, api_1.gql) `
      mutation stagedUploadsCreate($input: [StagedUploadInput!]!) {
        stagedUploadsCreate(input: $input) {
          userErrors {
            field
            message
          }
          stagedTargets {
            url
            resourceUrl
            parameters {
              name
              value
            }
          }
        }
      }
    `;
        const res = await api_1.graphqlClient.request(query, {
            input: [
                {
                    resource: "BULK_MUTATION_VARIABLES",
                    filename: "bulk_op_vars",
                    mimeType: "text/jsonl",
                    httpMethod: "POST",
                },
            ],
        });
        return res.stagedUploadsCreate.stagedTargets[0];
    }
    catch (err) {
        throw err;
    }
}
exports.generateJSONLUploadOptions = generateJSONLUploadOptions;
async function getBulkUpdateStatus() {
    return getBulkOperationStatus("MUTATION");
}
exports.getBulkUpdateStatus = getBulkUpdateStatus;
async function getBulkQueryStatus() {
    return getBulkOperationStatus("QUERY");
}
exports.getBulkQueryStatus = getBulkQueryStatus;
async function getBulkOperationStatus(type) {
    try {
        const query = (0, api_1.gql) `
      query {
        currentBulkOperation(type: ${type}) {
          id
          status
          errorCode
          objectCount
          url
        }
      }
    `;
        const res = await api_1.graphqlClient.request(query);
        const BusyStates = ["RUNNING", "CREATED", "CANCELING"];
        return { ...res.currentBulkOperation, BusyStates };
    }
    catch (error) {
        throw error;
    }
}
exports.getBulkOperationStatus = getBulkOperationStatus;
