"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getShopifyDate = exports.getFulfilledResults = exports.getThrottleResValues = exports.throttle = exports.formartId = void 0;
const promises_1 = require("timers/promises");
function formartId({ id, type }) {
    const base = "gid://shopify";
    const map = {
        PRODUCT_ID: `${base}/Product`,
        PRODUCT_VARIANT_ID: `${base}/ProductVariant`,
        INVENTORY_ITEM: `${base}/InventoryItem`,
        LOCATION: `${base}/Location`,
    };
    return `${map[type]}/${id}`;
}
exports.formartId = formartId;
async function throttle(req, reqInputList, limit = 4, wait = 2000) {
    const reqInput = [...reqInputList];
    const results = [];
    while (reqInput.length) {
        const tempInput = [];
        while (tempInput.length < limit + 1 && reqInput.length) {
            const val = reqInput.pop();
            if (val)
                tempInput.push(val);
        }
        const res2 = await Promise.allSettled(tempInput.map((input) => req(input)));
        results.push(...res2);
        if (wait === 0)
            continue;
        await (0, promises_1.setTimeout)(wait);
    }
    return results;
}
exports.throttle = throttle;
function getThrottleResValues(res) {
    return getFulfilledResults(res).map(({ value }) => value);
}
exports.getThrottleResValues = getThrottleResValues;
function getFulfilledResults(res) {
    return res.filter(({ status }) => status === "fulfilled");
}
exports.getFulfilledResults = getFulfilledResults;
function getShopifyDate(daysFromToday = 0) {
    const today = new Date();
    const date = new Date(new Date().setDate(today.getDate() + daysFromToday));
    return date.toISOString().split("T")[0];
}
exports.getShopifyDate = getShopifyDate;
